# Getestet mit Python 3.11.6 und PySide 6.6

import sys
import locale
import PySide6.QtWidgets as widgets
import PySide6.QtUiTools as uic
import PySide6.QtGui as gui
import PySide6.QtCore as core

# Python-Magie: Baue ein Dictionary der Nummern und Namen der Ereignistypen
event_names = {getattr(core.QEvent.Type, name): name for name in dir(core.QEvent.Type) if not name.startswith("__")}

# ein paar bunte Farben zum Zeichnen
colors = [core.Qt.GlobalColor.darkYellow, core.Qt.GlobalColor.magenta, core.Qt.GlobalColor.darkGreen, core.Qt.GlobalColor.cyan]
color_index = 0

class MyWidget(widgets.QWidget):
    def __init__(self, parent: widgets.QWidget):
        super().__init__(parent)
        self.setMouseTracking(True) # damit Mouse-Move-Events kommen, auch wenn keine Maustaste gedrückt ist

    # Diese Methode erhält _jedes_ Ereignis für das Widget. Man wird sie nur selten überschreiben.
    # Hier ist sie überschrieben, damit man die Ereignisse auf der Textkonsole mitlesen kann.
    def event(self, event: core.QEvent):
        print(event_names[event.type()])
        return super().event(event) # Abschließend ausführen, was diese Methode in der Mutterklasse macht.
    
    def paintEvent(self, event: gui.QPaintEvent) -> None:
        super().paintEvent(event) # Falls die Mutterklasse etwas zeichnen will. In diesem Fall ein überflüssiger Merkposten.

        painter = gui.QPainter(self) # der "Grafikkontext", ein Objekt mit den Zeichenbefehlen

        global color_index
        pen = gui.QPen(colors[color_index])
        color_index = (color_index + 1) % len(colors)
        pen.setWidth(10)
        painter.setPen(pen)
        painter.drawLine(0, 0, self.width(), self.height())
        painter.drawLine(0, self.height(), self.width(), 0)

        painter.end()

    # Erste Art der Ereignisbehandlung: eine überschriebene Methode
    def mousePressEvent(self, event: gui.QMouseEvent) -> None:
        print('Kuckuck!')

        # Demo der Statusleiste
        self.window().statusBar().showMessage(f"Der Farbindex war soeben gleich {color_index}.")

        # Erste Art, einen Menüeintrag auszugrauen (und gleichzeitig dessen Shortcuts abzustellen)
        self.window().actionsdfsdfsdf_sdfsd.setEnabled(False)

        return super().mousePressEvent(event)

app = widgets.QApplication(sys.argv)

# Damit "Cancel", "Ctrl" usw. in der Systemsprache erscheinen:
qt_base_translator = core.QTranslator()
language = locale.getdefaultlocale()[0].split("_")[0]
# Dies wäre eleganter (weil in PySide selbst formuliert),
# gibt aber im Zweifelsfall den Wert der Umgebungsvariablen LANG, was nicht die Systemsprache sein muss:
# language = core.QLocale.system().name().split("_")[0]
if qt_base_translator.load("qtbase_" + language, core.QLibraryInfo.path(core.QLibraryInfo.LibraryPath.TranslationsPath)):
   app.installTranslator(qt_base_translator)

loader = uic.QUiLoader()
loader.registerCustomWidget(MyWidget)
w = loader.load("events.ui")

# So geht es in PySide6 (anders als in PyQt5 aus den alten Videos) nicht mehr:
# ohne Ableiten nachträglich eine Methode ändern.
# def handleEvent(event: core.QEvent):
#     print(event_names[event.type()])
#     return True
# w.widget.event = handleEvent

def mach_was():
    print("Hallo")
    w.widget.update()

def erledige_den_Job():
    print("Hier müsste ich arbeiten.")
    app.quit()

# Zweite Art der Ereignisbehandlung: auszuführende Funktionen anmelden.
# Das geht, ohne die Klassen zu ändern -- anders als bei der ersten Art der Erieignisbehandlung (siehe oben).
# Allerdings sind nicht alle Arten an Ereignissen so verfügbar.
w.actionjhtfj_ztfjztfjzutf.triggered.connect(mach_was)
w.pushButton.clicked.connect(mach_was)
w.buttonBox.accepted.connect(erledige_den_Job)
w.buttonBox.rejected.connect(app.quit)

def bereite_menü_vor():
    w.actionsdfsdfsdf_sdfsdf.setEnabled(color_index % 2 == 0)

# Zweite Art, einen Menüeintrag auszugrauen: just in time, gerade bevor das Menü aufgeht
w.menusdf_sdfsdf.aboutToShow.connect(bereite_menü_vor)

timer = core.QTimer()
# timer.setSingleShot(True)
timer.timeout.connect(mach_was)
timer.start(500)

w.show()
sys.exit(app.exec())